# encoding: utf-8
#
# Copyright 2017 Jet Digital Inc. All rights reserved.
#------------------------------------------------------------------------------------------------

from jetstream.global_constants import Constants
from jetstream.global_constants import Permissions

# Disable no-member checks, as Constants class is dynamically constructed. "ignored-classes" would be better, but it doesn't work as inline option.
# pylint: disable=E1101

#: Version of the API spoken by this Python package.
API_VERSION = Constants.api_version

#: Version of JetStream this Python package came from.
BUILD_VERSION = (Constants.version_major, Constants.version_minor, Constants.version_subminor)


#: By default, the API will attempt to connect to a server running on 'localhost'.
DEFAULT_SERVER_ADDRESS = 'localhost'

#: By default, the API will connect to a server using the following TCP control port.
DEFAULT_API_PORT = Constants.default_api_port

#: By default, connection attempts will timeout and fail if unresponsive for 10 seconds.
DEFAULT_TIMEOUT_SECONDS = 10


#: Manifest flags: Identify a manifest mapping item as a regular file.
FILETYPE_REGULAR_FILE = Constants.file_type_regular_file

#: Manifest flags: Identify a manifest mapping item as a directory.
FILETYPE_DIRECTORY = Constants.file_type_directory


#: Symbolic Links: Ignore (do not copy) symbolic links.
SYMLINKS_IGNORE = Constants.symlinks_ignore

#: Symbolic Links: Follow the symlink and copy the file contents.
SYMLINKS_COPY_CONTENTS = Constants.symlinks_copy_contents


#: Transfer flags: Enable ``read`` flag.
#: Transfers with this flag will skip writing to disk at destination, and will
#: only read at sender side. This is most useful for performance and network debugging.
TRANSFER_FLAG_READ = Constants.transfer_flags_read

#: Transfer flags: Enable ``write`` flag. Transfers with this flag will will skip
#: reading from disk, and will only write at the destination. This is most useful
#: for performance and network debugging.
TRANSFER_FLAG_WRITE = Constants.transfer_flags_write

#: Transfer flags: Disable both read and write flags. This is most useful for
#: performance and network debugging.
TRANSFER_FLAG_NONE = Constants.transfer_flags_none

#: Transfer flags: Enable both ``read`` and ``write`` flags. This is the default value.
TRANSFER_FLAG_READ_WRITE = Constants.transfer_flags_read_write

#: Transfer flags: Set the modification time on the destination to match the source.
TRANSFER_FLAG_WRITE_MTIME = Constants.transfer_flags_write_mtime

#: Transfer flags: Don't write the file if the destination file exists and is newer than the source.
TRANSFER_FLAG_WRITE_ONLY_NEWER = Constants.transfer_flags_write_only_newer


#: Overwrite Mode: Overwrite files only if the existing file is same or zero (0) size.
OVERWRITE_IN_PLACE_SAME_SIZE = Constants.overwrite_in_place_same_size

#: Overwrite Mode: Use partfiles, and never overwrite existing files. Instead of
#: overwriting, the partfile will be left in place.
OVERWRITE_NEVER = Constants.overwrite_never

#: Overwrite Mode: Use partfiles, and overwrite existing files.
OVERWRITE_ALWAYS = Constants.overwrite_always

#: Overwrite Mode: Use partfiles, and overwrite existing files. The original file
#: will be backed up with a ``.jetbak`` extension.
OVERWRITE_VERSIONED = Constants.overwrite_versioned


#: Min Cipher: No encryption or validation. Data can viewed and changed by
#: a third party. Should not be used for production purposes.
MIN_CIPHER_NONE = Constants.min_cipher_none

#: Min Cipher: Data is not encrypted, but is still authenticated. Data can be
#: viewed but not altered or forged by a third-party.
MIN_CIPHER_UNENCRYPTED = Constants.min_cipher_unencrypted

#: Min Cipher: Data is encrypted and authenticated with strong 128-bit security.
MIN_CIPHER_ENCRYPTED_128 = Constants.min_cipher_encrypted_128

#: Min Cipher: Data is encrypted and authenticated with strong 256-bit security.
#: Encryption is stronger than the 128-bit version, but requires around 20% more
#: processing power.
MIN_CIPHER_ENCRYPTED_256 = Constants.min_cipher_encrypted_256

#: Fine grained API permissions.
PERMISSIONS = Permissions


#: The NULL type is used in the API to indicate an undefined or invalid argument,
#: disambiguating where None is an accepted value.
class Null(object):
    def __repr__(self):
        return 'NULL'

NULL = Null()
